library ieee;
  use ieee.std_logic_1164.all;
  use ieee.std_logic_unsigned.all;
  use ieee.numeric_std.all;
  
--  library UNISIM;
--  use UNISIM.Vcomponents.all;

entity video_signal_generator is
port (
    RESET       	: in    std_logic;
	 PIXEL_CLOCK	: in    std_logic;
    --
    VIDEO_X			: out   std_logic_vector(10 downto 0) := (others => '0');
    VIDEO_Y    	: out   std_logic_vector(10 downto 0) := (others => '0');
	 BLANK       	: out   std_logic;
    HSYNC       	: out   std_logic;
    VSYNC       	: out   std_logic;
	 ISYNC       	: out   std_logic
    );
end;


architecture a of video_signal_generator is

  constant H_FRONT_PORCH_START	: std_logic_vector (11 downto 0) := x"500";
  constant H_SYNC_START				: std_logic_vector (11 downto 0) := x"530";
  constant H_BACK_PORCH_START		: std_logic_vector (11 downto 0) := x"5A0";
  constant PIXEL_PER_LINE			: std_logic_vector (11 downto 0) := x"698";

  constant V_FRONT_PORCH_START 	: std_logic_vector (11 downto 0) := x"400";
  constant V_SYNC_START    		: std_logic_vector (11 downto 0) := x"401";
  constant V_BACK_PORCH_START  	: std_logic_vector (11 downto 0) := x"404";
  constant LINE_PER_FRAME      	: std_logic_vector (11 downto 0) := x"42A";
  

  --Signals
  signal pixel_count			: std_logic_vector(10 downto 0) := (others => '0');
  signal line_count			: std_logic_vector(10 downto 0) := (others => '0');
  signal pixel_count_d1		: std_logic_vector(10 downto 0) := (others => '0');
  signal line_count_d1		: std_logic_vector(10 downto 0) := (others => '0');
  
  signal v_sync				: std_logic := '0';
  signal h_sync				: std_logic := '0';
  signal i_sync				: std_logic := '0';
  signal v_sync_d1			: std_logic := '0';
  signal h_sync_d1			: std_logic := '0';
  signal i_sync_d1			: std_logic := '0';
  
  signal hor_end				: boolean;
  signal ver_end				: boolean;


  
  signal v_blank				: std_logic := '0';
  signal h_blank  			: std_logic := '0';
  signal i_blank  			: std_logic := '0';
  signal i_blank_d1 			: std_logic := '0';
  
begin


	hor_end <= (pixel_count = (PIXEL_PER_LINE(10 downto 0) - "1"));
	ver_end <= ( line_count = (LINE_PER_FRAME(10 downto 0) - "1"));


p_h_cnt : process(PIXEL_CLOCK)
begin

	if rising_edge(PIXEL_CLOCK) then
		if (RESET = '1') then
			pixel_count <= (others => '0');
		
		else
			if hor_end then
				pixel_count <= (others => '0');
			else
				pixel_count <= pixel_count + "1";
			end if;
		end if;
		
	end if;	 

end process;


p_v_cnt: process(PIXEL_CLOCK)
begin
	if rising_edge(PIXEL_CLOCK) then
	
		if (RESET = '1') then
			line_count <= (others => '0');
			
		else
			if hor_end then
				if ver_end then
					line_count <= (others => '0');
				else
					line_count <= line_count + "1";
				end if;
			end if;
			
		end if;
		
   end if;
end process;


  
mk_h_sync : process(PIXEL_CLOCK)

	variable hcnt_is_front_porch_start     : boolean;
	variable hcnt_is_sync_start           	: boolean;
	variable hcnt_is_back_porch_start      : boolean;
	
begin
	if rising_edge(PIXEL_CLOCK) then
		
		if (RESET = '1') then
			h_sync <= '1';
			h_blank <= '1';
			
		else		
			hcnt_is_front_porch_start     := (pixel_count = ( H_FRONT_PORCH_START(10 downto 0) - "1"));
			hcnt_is_sync_start            := (pixel_count = (        H_SYNC_START(10 downto 0) - "1"));
			hcnt_is_back_porch_start      := (pixel_count = (  H_BACK_PORCH_START(10 downto 0) - "1"));

			if hcnt_is_sync_start then
				h_sync <= '0';
			elsif hcnt_is_back_porch_start then
				h_sync <= '1';
			end if;

			if hcnt_is_front_porch_start then
				h_blank <= '1';
			elsif hor_end then
				h_blank <= '0';
			end if;
		end if;
		
	end if;
end process;



mk_v_sync : process(PIXEL_CLOCK)

	variable vcnt_is_front_porch_start : boolean;
	variable vcnt_is_sync_start        : boolean;
	variable vcnt_is_back_porch_start  : boolean;

begin

	if rising_edge(PIXEL_CLOCK) then

		if (RESET = '1') then
			v_sync <= '1';
			v_blank <= '1';
	
		else
	
			vcnt_is_front_porch_start		:= (line_count = (V_FRONT_PORCH_START(10 downto 0) - "1"));
			vcnt_is_sync_start       		:= (line_count = (       V_SYNC_START(10 downto 0) - "1"));
			vcnt_is_back_porch_start		:= (line_count = ( V_BACK_PORCH_START(10 downto 0) - "1"));

			if vcnt_is_sync_start and hor_end then
				v_sync <= '0';
			elsif vcnt_is_back_porch_start and hor_end then
				v_sync <= '1';
			end if;

			if vcnt_is_front_porch_start and hor_end then
				v_blank <= '1';
			elsif ver_end and hor_end then
				v_blank <= '0';
			end if;

		end if;
		
	end if;
end process;


mk_i_sync : process(PIXEL_CLOCK)

begin

	if rising_edge(PIXEL_CLOCK) then

		if (RESET = '1') then
			i_sync <= '0';
	
		elsif ver_end and hor_end then
			i_sync <= '1';
		else
			i_sync <= '0';
		end if;
		
	end if;
end process;



i_blank <= h_blank or v_blank;



synch_out : process(PIXEL_CLOCK)
begin
	if rising_edge (PIXEL_CLOCK) then
	
		if (RESET = '1') then
			HSYNC <= '1';
			VSYNC <= '1';
			ISYNC <= '1';

			BLANK <= '1';
			
			VIDEO_X <= (others => '0');
			VIDEO_Y <= (others => '0');

			
			-- add register level to go with 135 MHz
			h_sync_d1		<= '1';
			v_sync_d1 		<= '1';
			i_sync_d1 		<= '1';
			
			i_blank_d1 		<= '1';
			
			pixel_count_d1	<= (others => '0');
			line_count_d1	<= (others => '0');
			
			
		else
			-- add register level to go with 135 MHz		
			h_sync_d1		<= h_sync;
			v_sync_d1 		<= v_sync;
			i_sync_d1 		<= i_sync;
			
			i_blank_d1 		<= i_blank;
			
			pixel_count_d1	<= pixel_count;
			line_count_d1	<= line_count;

			
			HSYNC		<= h_sync_d1;
			VSYNC 	<= v_sync_d1;
			ISYNC 	<= i_sync_d1;
			
			BLANK 	<= i_blank_d1;
			
			VIDEO_X	<= pixel_count_d1;
			VIDEO_Y	<= line_count_d1;
			
		end if;		

	end if;

end process;
  
end architecture a;

